@@ -28,7 +28,6 @@ from pytorch_lightning.callbacks import Callback
 from torch import Tensor
 from torch.utils.data import DataLoader, Dataset
 
-from warp_drive.managers.function_manager import CUDASampler
 from warp_drive.training.algorithms.a2c import A2C
 from warp_drive.training.algorithms.ppo import PPO
 from warp_drive.training.models.fully_connected import FullyConnected
@@ -242,7 +241,22 @@ class WarpDriveModule(LightningModule):
             push_data_batch_placeholders=False,
         )
 
-        self.cuda_sample_controller = CUDASampler(self.cuda_envs.cuda_function_manager)
+        if env_wrapper.env_backend == "pycuda":
+            from warp_drive.managers.pycuda_managers.pycuda_function_manager import (
+                PyCUDASampler,
+            )
+
+            self.cuda_sample_controller = PyCUDASampler(
+                self.cuda_envs.cuda_function_manager
+            )
+        elif env_wrapper.env_backend == "numba":
+            from warp_drive.managers.numba_managers.numba_function_manager import (
+                NumbaSampler,
+            )
+
+            self.cuda_sample_controller = NumbaSampler(
+                self.cuda_envs.cuda_function_manager
+            )
 
         # Register action placeholders
         if self.create_separate_placeholders_for_each_policy:
@@ -956,16 +970,31 @@ class PerfStatsCallback(Callback):
         self.iters = 0
         self.steps = 0
         self.training_time = 0.0
+        self.total_time = 0.0
 
         # For timing purposes
-        self.start_event = torch.cuda.Event(enable_timing=True)
-        self.end_event = torch.cuda.Event(enable_timing=True)
+        self.start_event_batch = torch.cuda.Event(enable_timing=True)
+        self.end_event_batch = torch.cuda.Event(enable_timing=True)
 
-    def get_perf_stats(self):
-        return {
+        self.start_event_fit = torch.cuda.Event(enable_timing=True)
+        self.end_event_fit = torch.cuda.Event(enable_timing=True)
+
+    def get_perf_stats(self, with_fit=False):
+        perf_stats = {
             "Mean training time per iter (ms)": self.training_time * 1000 / self.iters,
-            "Mean steps per sec (training time)": self.steps / self.training_time,
+            "Mean steps per sec (training)": self.steps / self.training_time,
         }
+        if with_fit:
+            # Add the stats obtained at the end of the trainer.fit() call
+            perf_stats.update(
+                {
+                    "Mean total time per iter (ms)": self.total_time
+                    * 1000
+                    / self.iters,
+                    "Mean steps per sec (total)": self.steps / self.total_time,
+                }
+            )
+        return perf_stats
 
     def pretty_print(self, stats):
         print("=" * 40)
@@ -984,19 +1013,36 @@ class PerfStatsCallback(Callback):
         assert pl_module is not None
         self.iters += 1
         self.steps = self.iters * self.batch_size
-        self.start_event.record()
+        self.start_event_batch.record()
 
     def on_batch_end(self, trainer=None, pl_module=None):
         assert trainer is not None
         assert pl_module is not None
-        self.end_event.record()
+        self.end_event_batch.record()
         torch.cuda.synchronize()
 
-        self.training_time += self.start_event.elapsed_time(self.end_event) / 1000
+        self.training_time += (
+            self.start_event_batch.elapsed_time(self.end_event_batch) / 1000
+        )
 
         if self.iters % self.log_freq == 0 or self.iters == self.num_iters:
             self.pretty_print(self.get_perf_stats())
 
+    def on_fit_start(self, trainer=None, pl_module=None):
+        assert trainer is not None
+        assert pl_module is not None
+        self.start_event_fit.record()
+
+    def on_fit_end(self, trainer=None, pl_module=None):
+        assert trainer is not None
+        assert pl_module is not None
+        self.end_event_fit.record()
+        torch.cuda.synchronize()
+
+        self.total_time += self.start_event_fit.elapsed_time(self.end_event_fit) / 1000
+        if self.iters % self.log_freq == 0 or self.iters == self.num_iters:
+            self.pretty_print(self.get_perf_stats(with_fit=True))
+
 
 class CUDACallback(Callback):
     """
